<?php
namespace Mkb\Sdk;

use Mkb\Sdk\Exceptions\InvalidPropertyValueException;

class Receipt
{
	const TYPE_SELL   = 0;
	const TYPE_REFUND = 1;

	private $items         = [];
	private $resultTotal   = 0;
	private $currentSum    = 0;
	private $sno           = '';
	private $paymentMethod = '';

	public function __construct($sno, $paymentMethod, $total = 0.0)
	{
		$this->resultTotal   = intval(round($total * 100));
		$this->sno           = $sno;
		$this->paymentMethod = $paymentMethod;
	}

	public function addItem($name, $amount, $vatId, $paymentObject = 'commodity', $quantity = 1.0)
	{
		if ($amount <= 0) {
			return;
		}

		if ($quantity == 0) {
			return;
		}

		$vatSum = $this->calcVat($vatId, $amount * $quantity);

		$this->items[] = array(
			"quantity" => round($quantity * 1000),
			"price"    => round($amount * 100),
			"sum"      => round($amount * $quantity * 100),
			"vat"      => [
				'type' => $vatId,
				'sum'  => $vatSum,
			],
			"name"     => mb_substr($name, 0, 128, "utf-8"),
			"object"   => $paymentObject,
			"method"   => $this->paymentMethod,
		);
		$this->currentSum += $amount * 100 * $quantity;
	}

	private function normalize()
	{
		$this->currentSum = round($this->currentSum);
		if ($this->resultTotal != 0 && $this->resultTotal != $this->currentSum) {
			$coefficient = $this->resultTotal / $this->currentSum;
			$realprice   = 0;
			$aloneId     = null;
			foreach ($this->items as $index => &$item) {
				$item['price'] = round($coefficient * $item['price']);
				$realprice += $item['price'] * $item['quantity'] / 1000;
				if ($aloneId === null && $item['quantity'] === 1000) {
					$aloneId = $index;
				}

			}
			unset($item);
			if ($aloneId === null) {
				foreach ($this->items as $index => $item) {
					if ($aloneId === null && $item['quantity'] > 1000) {
						$aloneId = $index;
						break;
					}
				}
			}
			if ($aloneId === null) {
				$aloneId = 0;
			}

			$realprice = round($realprice);

			$diff = $this->resultTotal - $realprice;
			if (abs($diff) >= 0.001) {
				if ($this->items[$aloneId]['quantity'] === 1000) {
					$this->items[$aloneId]['price'] = round($this->items[$aloneId]['price'] + $diff);
				} elseif (
					count($this->items) == 1
					&& abs(round($this->resultTotal / $this->items[$aloneId]['quantity']) - $this->resultTotal / $this->items[$aloneId]['quantity']) < 0.001
				) {
					$this->items[$aloneId]['price'] = round($this->resultTotal / $this->items[$aloneId]['quantity']);
				} elseif ($this->items[$aloneId]['quantity'] > 1000) {
					$tmpItem = $this->items[$aloneId];
					$vatSum  = $this->calcVat($tmpItem['vat']['type'], round($tmpItem['price'] + $diff));
					$item    = array(
						"quantity"       => 1000,
						"price"          => round($tmpItem['price'] + $diff),
						'sum'            => round($tmpItem['price'] + $diff),
						"vat"            => [
							'type' => $tmpItem['vat']['type'],
							'sum'  => $vatSum,
						],
						"name"           => $tmpItem['name'],
						"payment_object" => $tmpItem['payment_object'],
						"payment_method" => $tmpItem['payment_method'],
					);
					$this->items[$aloneId]['quantity'] -= 1000;
					array_splice($this->items, $aloneId + 1, 0, array($item));
				} else {
					$this->items[$aloneId]['price'] = round($this->items[$aloneId]['price'] + $diff / ($this->items[$aloneId]['quantity'] / 1000));
				}
				$this->items[$aloneId]['sum']        = $this->items[$aloneId]['price'] * $this->items[$aloneId]['quantity'] / 1000;
				$this->items[$aloneId]['vat']['sum'] = $this->calcVat($this->items[$aloneId]['vat']['type'], $this->items[$aloneId]['sum']);
			}
		}
	}

	private function correctDimmensoin()
	{
		foreach ($this->items as &$item) {
			$item['quantity']   = number_format(round($item['quantity'] / 1000, 3), 3, '.', '');
			$item['price']      = number_format(round($item['price'] / 100, 2), 2, '.', '');
			$item['sum']        = number_format(round($item['sum'] / 100, 2), 2, '.', '');
			$item['vat']['sum'] = number_format(round($item['vat']['sum'] / 100, 2), 2, '.', '');
		}
	}

	private function calcVat($vatId, $sum)
	{
		switch ($vatId) {
			case 'none':
			case 'vat0':
				return 0;
				break;
			case 'vat10':
				return round($sum * 0.10);
				break;
			case 'vat20':
				return round($sum * 0.20);
				break;
			case 'vat110':
				return round($sum * 10 / 110);
				break;
			case 'vat120':
				return round($sum * 20 / 120);
				break;
			default:
				throw new InvalidPropertyValueException('Vat must be one of "none, vat0, vat10, vat20, vat110, vat120"', 0, 'VatId', $vatId);
		}
	}

	public function getItems()
	{
		$this->normalize();
		$this->correctDimmensoin();
		return $this->items;
	}
}
